package com.jiuqi.syntax;

import java.util.GregorianCalendar;
import java.util.Date;
import java.math.BigDecimal;
import java.util.Calendar;

public class CommonData implements Cloneable
{
    public static int CD_NEW_COUNT;
    public static int CD_LIVE_COUNT;
    public static final int NULL_TYPE = 0;
    public static final int INT_TYPE = 1;
    public static final int REAL_TYPE = 2;
    public static final int STRING_TYPE = 3;
    public static final int EXP_TYPE = 4;
    public static final int BOOL_TYPE = 5;
    public static final int DATE_TYPE = 6;
    public static final int MEMO_TYPE = 7;
    public static final int OBJECT_TYPE = 8;
    public static final int DECIMAL_TYPE = 10;
    public int type;
    public boolean isNull;
    public int cdInt;
    public double cdReal;
    public String cdString;
    public boolean cdBool;
    public Calendar cdDate;
    public Object cdObject;
    public BigDecimal cdDecimal;
    protected static final DataOperator[] operators;
    
    static {
        CommonData.CD_NEW_COUNT = 0;
        CommonData.CD_LIVE_COUNT = 0;
        (operators = new DataOperator[16])[0] = new DataOperator() {
            @Override
            public int compare(final CommonData v1, final CommonData v2, final double delta) {
                return 0;
            }
        };
        CommonData.operators[1] = new DataOperator() {
            @Override
            public int compare(final CommonData v1, final CommonData v2, final double delta) {
                return v1.cdInt - v2.cdInt;
            }
        };
        CommonData.operators[2] = new DataOperator() {
            @Override
            public int compare(final CommonData v1, final CommonData v2, final double delta) {
                if (delta == 0.0) {
                    if (Math.abs(v1.cdReal - v2.cdReal) < 1.0E-5) {
                        return 0;
                    }
                    if (v1.cdReal > v2.cdReal) {
                        return 1;
                    }
                    return -1;
                }
                else {
                    if (Math.abs(v1.cdReal - v2.cdReal) < delta) {
                        return 0;
                    }
                    if (v1.cdReal > v2.cdReal) {
                        return 1;
                    }
                    return -1;
                }
            }
        };
        CommonData.operators[3] = new DataOperator() {
            @Override
            public int compare(final CommonData v1, final CommonData v2, final double delta) {
                return v1.cdString.compareTo(v2.cdString);
            }
        };
        CommonData.operators[4] = CommonData.operators[3];
        CommonData.operators[5] = new DataOperator() {
            @Override
            public int compare(final CommonData v1, final CommonData v2, final double delta) {
                return (v1.cdBool == v2.cdBool) ? 0 : 255;
            }
        };
        CommonData.operators[6] = new DataOperator() {
            @Override
            public int compare(final CommonData v1, final CommonData v2, final double delta) {
                if (v1.cdDate == null || v2.cdDate == null) {
                    return 255;
                }
                if (v1.cdDate.before(v2.cdDate)) {
                    return -1;
                }
                if (v1.cdDate.after(v2.cdDate)) {
                    return 1;
                }
                return 0;
            }
        };
        CommonData.operators[10] = new DataOperator() {
            @Override
            public int compare(final CommonData v1, final CommonData v2, double delta) {
                if (v1.cdDecimal == null || v2.cdDecimal == null) {
                    return 255;
                }
                if (delta == 0.0) {
                    delta = 1.0E-5;
                }
                final BigDecimal comResult = v1.cdDecimal.subtract(v2.cdDecimal);
                if (comResult.abs().compareTo(new BigDecimal(delta)) < 0) {
                    return 0;
                }
                if (comResult.signum() > 0) {
                    return 1;
                }
                return -1;
            }
        };
    }
    
    public double getIntRealValue() {
        if (this.type == 1) {
            return this.cdInt;
        }
        if (this.type == 2) {
            return this.cdReal;
        }
        return 0.0;
    }
    
    public CommonData() {
        this.type = 0;
        this.isNull = false;
        this.cdInt = 0;
        this.cdReal = 0.0;
        this.cdString = "";
        this.cdBool = false;
        this.cdDate = null;
        this.cdObject = null;
        this.cdDecimal = null;
        ++CommonData.CD_NEW_COUNT;
        ++CommonData.CD_LIVE_COUNT;
    }
    
    public Object clone() {
        final CommonData d = new CommonData();
        this.assignTo(d);
        return d;
    }
    
    public void assign(final CommonData source) {
        this.type = source.type;
        this.isNull = source.isNull;
        this.initNullValue();
        if (!this.isNull) {
            switch (this.type) {
                case 1: {
                    this.cdInt = source.cdInt;
                    break;
                }
                case 2: {
                    this.cdReal = source.cdReal;
                    break;
                }
                case 3: {
                    this.cdString = source.cdString;
                    break;
                }
                case 4: {
                    this.cdString = source.cdString;
                    break;
                }
                case 5: {
                    this.cdBool = source.cdBool;
                    break;
                }
                case 6: {
                    this.cdDate = source.cdDate;
                    break;
                }
                case 7: {
                    this.cdObject = source.cdObject;
                }
                case 8: {
                    this.cdObject = source.cdObject;
                    break;
                }
                case 10: {
                    this.cdDecimal = source.cdDecimal;
                    break;
                }
            }
        }
    }
    
    public void assignTo(final CommonData dest) {
        dest.assign(this);
    }
    
    public void initValue() {
        switch (this.type) {
            case 1: {
                this.cdInt = 0;
                break;
            }
            case 2: {
                this.cdReal = 0.0;
                break;
            }
            case 3: {
                this.cdString = "";
                break;
            }
            case 4: {
                this.cdString = "";
                break;
            }
            case 5: {
                this.cdBool = false;
                break;
            }
            case 6: {
                this.cdDate = null;
                break;
            }
            case 7: {
                this.cdObject = null;
                break;
            }
            case 8: {
                this.cdObject = null;
                break;
            }
            case 10: {
                this.cdDecimal = new BigDecimal(0);
                break;
            }
        }
    }
    
    public void initNullValue() {
        if (this.isNull) {
            this.initValue();
        }
    }
    
    public int compareTo(final CommonData other) {
        return this.compareTo(other, 0.0);
    }
    
    public final int compareTo(final CommonData other, final double diffValue) {
        if (other == null) {
            return 255;
        }
        if (this.type != other.type) {
            return 255;
        }
        if (this.isNull && other.isNull) {
            return 0;
        }
        if (this.isNull != other.isNull) {
            this.initNullValue();
            other.initNullValue();
        }
        if (CommonData.operators[this.type] == null) {
            return 255;
        }
        return CommonData.operators[this.type].compare(this, other, diffValue);
    }
    
    public boolean equals(final CommonData other) {
        return this.compareTo(other, 0.0) == 0;
    }
    
    public boolean equals(final CommonData other, final double diffValue) {
        return this.compareTo(other, diffValue) == 0;
    }
    
    @Override
    public String toString() {
        String s = new String();
        switch (this.type) {
            case 0: {
                s = "";
                break;
            }
            case 1: {
                s = String.valueOf(this.cdInt);
                break;
            }
            case 2: {
                s = String.valueOf(this.cdReal);
                break;
            }
            case 3: {
                s = this.cdString;
                break;
            }
            case 4: {
                s = this.cdString;
                break;
            }
            case 5: {
                s = String.valueOf(this.cdBool);
                break;
            }
            case 6: {
                if (this.cdDate == null) {
                    s = "";
                    break;
                }
                s = this.cdDate.toString();
                break;
            }
            case 7: {
                s = "";
                break;
            }
            case 8: {
                s = "";
                break;
            }
            case 10: {
                if (this.cdDecimal == null) {
                    s = "";
                    break;
                }
                s = this.cdDecimal.toString();
                break;
            }
        }
        return s;
    }
    
    public void setDateValue(final Date dateValue) {
        this.type = 6;
        this.isNull = (dateValue == null);
        if (!this.isNull && this.cdDate == null) {
            this.cdDate = new GregorianCalendar();
        }
        if (dateValue != null) {
            this.cdDate.setTime(dateValue);
        }
        else if (this.cdDate != null) {
            this.cdDate.clear();
        }
    }
    
    public Calendar getValidcdDate() {
        if (this.cdDate == null) {
            this.cdDate = Calendar.getInstance();
            this.cdDate.getTime().setTime(0L);
            this.cdDate.set(1, 1899);
            this.cdDate.set(2, 1);
            this.cdDate.set(5, 1);
        }
        return this.cdDate;
    }
    
    @Override
    protected void finalize() throws Throwable {
        --CommonData.CD_LIVE_COUNT;
        super.finalize();
    }
    
    protected abstract static class DataOperator
    {
        public abstract int compare(final CommonData p0, final CommonData p1, final double p2);
    }
}
